@@ -12,15 +12,14 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #include <lttoolbox/compiler.h>
 #include <lttoolbox/compression.h>
 #include <lttoolbox/entry_token.h>
 #include <lttoolbox/lt_locale.h>
 #include <lttoolbox/xml_parse_util.h>
+#include <lttoolbox/string_to_wostream.h>
 
 #include <cstdlib>
 #include <iostream>
@@ -49,6 +48,7 @@ wstring const Compiler::COMPILER_SECTION_ELEM       = L"section";
 wstring const Compiler::COMPILER_ID_ATTR            = L"id";
 wstring const Compiler::COMPILER_TYPE_ATTR	    = L"type";
 wstring const Compiler::COMPILER_IDENTITY_ELEM      = L"i";
+wstring const Compiler::COMPILER_IDENTITYGROUP_ELEM = L"ig";
 wstring const Compiler::COMPILER_JOIN_ELEM	    = L"j";
 wstring const Compiler::COMPILER_BLANK_ELEM	    = L"b";
 wstring const Compiler::COMPILER_POSTGENERATOR_ELEM = L"a";
@@ -61,7 +61,11 @@ wstring const Compiler::COMPILER_V_ATTR             = L"v";
 wstring const Compiler::COMPILER_VL_ATTR            = L"vl";
 wstring const Compiler::COMPILER_VR_ATTR            = L"vr";
 
-Compiler::Compiler()
+Compiler::Compiler() :
+reader(0),
+verbose(false),
+first_element(false),
+acx_current_char(0)
 {
 }
 
@@ -77,7 +81,7 @@ Compiler::parseACX(string const &fichero, wstring const &dir)
     reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
     if(reader == NULL)
     {
-      cerr << "Error: cannot open '" << fichero << "'." << endl;
+      wcerr << "Error: cannot open '" << fichero << "'." << endl;
       exit(EXIT_FAILURE);
     }
     int ret = xmlTextReaderRead(reader);
@@ -96,7 +100,7 @@ Compiler::parse(string const &fichero, wstring const &dir)
   reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
   if(reader == NULL)
   {
-    cerr << "Error: Cannot open '" << fichero << "'." << endl;
+    wcerr << "Error: Cannot open '" << fichero << "'." << endl;
     exit(EXIT_FAILURE);
   }
 
@@ -412,7 +416,7 @@ Compiler::skip(wstring &name, wstring const &elem, bool open)
 }
 
 EntryToken
-Compiler::procIdentity()
+Compiler::procIdentity(bool ig)
 {
   list<int> both_sides;
 
@@ -424,7 +428,7 @@ Compiler::procIdentity()
     {
       xmlTextReaderRead(reader);
       name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
-      if(name == COMPILER_IDENTITY_ELEM)
+      if(name == COMPILER_IDENTITY_ELEM || name == COMPILER_IDENTITYGROUP_ELEM)
       {
         break;
       }
@@ -439,7 +443,17 @@ Compiler::procIdentity()
   }
   first_element = false; 
   EntryToken e;
-  e.setSingleTransduction(both_sides, both_sides);
+  if(ig)
+  {
+    list<int> right;
+    right.push_back(static_cast<int>(L'#'));
+    right.insert(right.end(), both_sides.begin(), both_sides.end());
+    e.setSingleTransduction(both_sides, right);
+  }
+  else
+  {
+    e.setSingleTransduction(both_sides, both_sides);
+  }
   return e;
 }
 
@@ -565,7 +579,7 @@ Compiler::insertEntryTokens(vector<EntryToken> const &elements)
   }
   else
   {
-    // compilacin de dictionary
+    // compilación de dictionary
 
     Transducer &t = sections[current_section];
     int e = t.getInitial();
@@ -673,7 +687,7 @@ Compiler::procEntry()
   wstring varl   = this->attrib(COMPILER_VL_ATTR);
   wstring varr   = this->attrib(COMPILER_VR_ATTR);
 
-  //if entry is masked by a restriction of direction or an ignore mark
+  // if entry is masked by a restriction of direction or an ignore mark
   if((atributo != L"" && atributo != direction) 
    || ignore == COMPILER_IGNORE_YES_VAL
    || (altval != L"" && altval != alt)
@@ -700,8 +714,8 @@ Compiler::procEntry()
     int ret = xmlTextReaderRead(reader);
     if(ret != 1)
     {
-      cerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-      cerr << L"): Parse error." << endl;
+      wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+      wcerr << L"): Parse error." << endl;
       exit(EXIT_FAILURE);
     }
     wstring name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
@@ -719,7 +733,11 @@ Compiler::procEntry()
     }
     else if(name == COMPILER_IDENTITY_ELEM)
     {
-      elements.push_back(procIdentity());
+      elements.push_back(procIdentity(false));
+    }
+    else if(name == COMPILER_IDENTITYGROUP_ELEM)
+    {
+      elements.push_back(procIdentity(true));
     }
     else if(name == COMPILER_REGEXP_ELEM)
     {
@@ -729,7 +747,7 @@ Compiler::procEntry()
     {
       elements.push_back(procPar());
 
-      // deteccin del uso de paradigmas no definidos
+      // detección del uso de paradigmas no definidos
 
       wstring const &p = elements.rbegin()->paradigmName();
 
@@ -739,7 +757,7 @@ Compiler::procEntry()
         wcerr << L"): Undefined paradigm '" << p << L"'." <<endl;
         exit(EXIT_FAILURE);
       }
-      // descartar entradas con paradigms vacos (por las direciones,
+      // descartar entradas con paradigms vacíos (por las direciones,
       // normalmente
       if(paradigms[p].isEmpty())
       {
@@ -810,7 +828,7 @@ Compiler::procNode()
   xmlChar const *xnombre = xmlTextReaderConstName(reader);
   wstring nombre = XMLParseUtil::towstring(xnombre);
 
-  // HACER: optimizar el orden de ejecucin de esta ristra de "ifs"
+  // HACER: optimizar el orden de ejecución de esta ristra de "ifs"
 
   if(nombre == L"#text")
   {
