@@ -26,7 +26,7 @@
 #include <libxml/encoding.h>
 
 #ifdef _WIN32
-#include <utf8_fwrap.hpp>
+#include <utf8_fwrap.h>
 #endif
 
 using namespace std;
@@ -47,7 +47,7 @@ Expander::expand(string const &fichero, FILE *output)
   reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
   if(reader == NULL)
   {
-    cerr << "Error: Cannot open '" << fichero << "'." << endl;
+    wcerr << "Error: Cannot open '" << fichero << "'." << endl;
     exit(EXIT_FAILURE);
   }
 
@@ -219,6 +219,35 @@ Expander::procIdentity()
   return both_sides;  
 }
 
+pair<wstring, wstring>
+Expander::procIdentityGroup()
+{
+  wstring lhs = L"";
+  wstring rhs = L"#";
+  wstring both_sides = L"";
+
+  if(!xmlTextReaderIsEmptyElement(reader))
+  {
+    wstring name = L"";
+
+    while(true)
+    {
+      xmlTextReaderRead(reader);
+      name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
+      if(name == Compiler::COMPILER_IDENTITYGROUP_ELEM)
+      {
+        break;
+      }
+      readString(both_sides, name);
+    }
+  }
+  lhs += both_sides;
+  rhs += both_sides;
+
+  pair<wstring, wstring> e(lhs, rhs);
+  return e;
+}
+
 pair<wstring, wstring>
 Expander::procTransduction()
 {
@@ -371,6 +400,13 @@ Expander::procEntry(FILE *output)
       append(items_lr, val);
       append(items_rl, val);
     }
+    else if(name == Compiler::COMPILER_IDENTITYGROUP_ELEM)
+    {      
+      pair<wstring, wstring> p = procIdentityGroup();
+      append(items, p);
+      append(items_lr, p);
+      append(items_rl, p);
+    }
     else if(name == Compiler::COMPILER_REGEXP_ELEM)
     {
       wstring val = L"__REGEXP__" + procRegexp();
