@@ -826,9 +826,32 @@ FSTProcessor::load(FILE *input)
   }
 }
 
+void
+FSTProcessor::lsx_wrapper_null_flush(FILE *input, FILE *output)
+{
+  setNullFlush(false);
+  //nullFlushGeneration = true;
+
+  while(!feof(input))
+  {
+    lsx(input, output);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0)
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
 void
 FSTProcessor::lsx(FILE *input, FILE *output)
 {
+  if(getNullFlush())
+  {
+    lsx_wrapper_null_flush(input, output);
+  }
+
   vector<State> new_states, alive_states;
   wstring blank, out, in, alt_out, alt_in;
   bool outOfWord = true;
@@ -839,7 +862,12 @@ FSTProcessor::lsx(FILE *input, FILE *output)
 
   while(!feof(input))
   {
-    int val = fgetwc(input);
+    int val = fgetwc_unlocked(input);
+
+    if (val == 0) {
+      blankqueue.push(blank);
+      break;
+    }
 
     if(val == L'+' && isEscaped(val) && !outOfWord)
     {
