@@ -30,7 +30,7 @@
 #endif
 
 #ifdef _WIN32
-#include <utf8_fwrap.hpp>
+#include <utf8_fwrap.h>
 #endif
 
 using namespace std;
@@ -38,26 +38,30 @@ using namespace std;
 void endProgram(char *name)
 {
   cout << basename(name) << ": process a stream with a letter transducer" << endl;
-  cout << "USAGE: " << basename(name) << " [ -a | -b | -c | -d | -e | -g | -n | -p | -s | -t | -v | -h -z -w ] fst_file [input_file [output_file]]" << endl;
+  cout << "USAGE: " << basename(name) << " [ -a | -b | -c | -d | -e | -g | -n | -p | -s | -t | -v | -h -z -w ] [ -i icx_file ] [ -r rcx_file ] fst_file [input_file [output_file]]" << endl;
   cout << "Options:" << endl;
 #if HAVE_GETOPT_LONG
-  cout << "  -a, --analysis:         morphological analysis (default behavior)" << endl;
-  cout << "  -b, --bilingual:        lexical transfer" << endl;
-  cout << "  -c, --case-sensitive:   use the literal case of the incoming characters" << endl;
-  cout << "  -d, --debugged-gen      morph. generation with all the stuff" <<endl;
-  cout << "  -e, --decompose-nouns:  Try to decompound unknown words" << endl;
-  cout << "  -g, --generation:       morphological generation" << endl;
-  cout << "  -l, --tagged-gen:       morphological generation keeping lexical forms" << endl;
-  cout << "  -m, --tagged-nm-gen:    same as -l but without unknown word marks" << endl;
-  cout << "  -n, --non-marked-gen    morph. generation without unknown word marks" << endl;
-  cout << "  -o, --surf-bilingual:   lexical transfer with surface forms" << endl;
-  cout << "  -p, --post-generation:  post-generation" << endl;
-  cout << "  -s, --sao:              SAO annotation system input processing" << endl;
-  cout << "  -t, --transliteration:  apply transliteration dictionary" << endl;
-  cout << "  -v, --version:          version" << endl;
-  cout << "  -z, --null-flush:       flush output on the null character " << endl;
-  cout << "  -w, --dictionary-case:  use dictionary case instead of surface case" << endl;
-  cout << "  -h, --help:             show this help" << endl;
+  cout << "  -a, --analysis:          morphological analysis (default behavior)" << endl;
+  cout << "  -b, --bilingual:         lexical transfer" << endl;
+  cout << "  -c, --case-sensitive:    use the literal case of the incoming characters" << endl;
+  cout << "  -d, --debugged-gen       morph. generation with all the stuff" << endl;
+  cout << "  -e, --decompose-nouns:   Try to decompound unknown words" << endl;
+  cout << "  -g, --generation:        morphological generation" << endl;
+  cout << "  -i, --ignored-chars:     specify file with characters to ignore" << endl;
+  cout << "  -r, --restore-chars:     specify file with characters to diacritic restoration" << endl;
+  cout << "  -l, --tagged-gen:        morphological generation keeping lexical forms" << endl;
+  cout << "  -m, --tagged-nm-gen:     same as -l but without unknown word marks" << endl;
+  cout << "  -n, --non-marked-gen     morph. generation without unknown word marks" << endl;
+  cout << "  -o, --surf-bilingual:    lexical transfer with surface forms" << endl;
+  cout << "  -p, --post-generation:   post-generation" << endl;
+  cout << "  -s, --sao:               SAO annotation system input processing" << endl;
+  cout << "  -t, --transliteration:   apply transliteration dictionary" << endl;
+  cout << "  -v, --version:           version" << endl;
+  cout << "  -z, --null-flush:        flush output on the null character " << endl;
+  cout << "  -w, --dictionary-case:   use dictionary case instead of surface case" << endl;
+  cout << "  -C, --careful-case:      use dictionary case if present, else surface" << endl;
+  cout << "  -I, --no-default-ignore: skips loading the default ignore characters" << endl;
+  cout << "  -h, --help:              show this help" << endl;
 #else
   cout << "  -a:   morphological analysis (default behavior)" << endl;
   cout << "  -b:   lexical transfer" << endl;
@@ -65,6 +69,8 @@ void endProgram(char *name)
   cout << "  -d:   morph. generation with all the stuff" << endl;
   cout << "  -e:   try to decompose unknown words as compounds" << endl;
   cout << "  -g:   morphological generation" << endl;
+  cout << "  -i:   specify file with characters to ignore" << endl;
+  cout << "  -r:   specify file with characters to diacritic restoration" << endl;
   cout << "  -l:   morphological generation keeping lexical forms" << endl;
   cout << "  -n:   morph. generation without unknown word marks" << endl;
   cout << "  -o:   lexical transfer with surface forms" << endl;
@@ -73,6 +79,8 @@ void endProgram(char *name)
   cout << "  -t:   apply transliteration dictionary" << endl;
   cout << "  -v:   version" << endl;
   cout << "  -z:   flush output on the null character " << endl;
+  cout << "  -C:   use dictionary case if present, else surface" << endl;
+  cout << "  -I:   skips loading the default ignore characters" << endl;
   cout << "  -w:   use dictionary case instead of surface case" << endl;
   cout << "  -h:   show this help" << endl;
 #endif
@@ -95,22 +103,26 @@ int main(int argc, char *argv[])
 #if HAVE_GETOPT_LONG
   static struct option long_options[]=
     {
-      {"analysis",        0, 0, 'a'},
-      {"bilingual",       0, 0, 'b'},
-      {"surf-bilingual",  0, 0, 'o'},
-      {"generation",      0, 0, 'g'},
-      {"non-marked-gen",  0, 0, 'n'},
-      {"debugged-gen",    0, 0, 'd'},
-      {"tagged-gen",      0, 0, 'l'},
-      {"tagged-nm-gen",   0, 0, 'm'},
-      {"post-generation", 0, 0, 'p'},
-      {"sao",             0, 0, 's'},
-      {"transliteration", 0, 0, 't'},
-      {"null-flush",      0, 0, 'z'},
-      {"dictionary-case", 0, 0, 'w'},
-      {"version",	  0, 0, 'v'},
-      {"case-sensitive",  0, 0, 'c'},
-      {"help",            0, 0, 'h'}
+      {"analysis",          0, 0, 'a'},
+      {"bilingual",         0, 0, 'b'},
+      {"surf-bilingual",    0, 0, 'o'},
+      {"generation",        0, 0, 'g'},
+      {"ignored-chars",     1, 0, 'i'},
+      {"restore-chars",     1, 0, 'i'},
+      {"non-marked-gen",    0, 0, 'n'},
+      {"debugged-gen",      0, 0, 'd'},
+      {"tagged-gen",        0, 0, 'l'},
+      {"tagged-nm-gen",     0, 0, 'm'},
+      {"post-generation",   0, 0, 'p'},
+      {"sao",               0, 0, 's'},
+      {"transliteration",   0, 0, 't'},
+      {"null-flush",        0, 0, 'z'},
+      {"dictionary-case",   0, 0, 'w'},
+      {"version",	    0, 0, 'v'},
+      {"case-sensitive",    0, 0, 'c'},
+      {"careful-case",      0, 0, 'C'},
+      {"no-default-ignore", 0, 0, 'I'},
+      {"help",              0, 0, 'h'}
     };
 #endif
 
@@ -118,9 +130,9 @@ int main(int argc, char *argv[])
   {
 #if HAVE_GETOPT_LONG
     int option_index;
-    int c = getopt_long(argc, argv, "abceglmndopstzwvh", long_options, &option_index);
+    int c = getopt_long(argc, argv, "abcegi:r:lmndopstzwvCIh", long_options, &option_index);
 #else
-    int c = getopt(argc, argv, "abceglmndopstzwvh");
+    int c = getopt(argc, argv, "abcegi:r:lmndopstzwvCIh");
 #endif
 
     if(c == -1)
@@ -134,7 +146,21 @@ int main(int argc, char *argv[])
       fstp.setCaseSensitiveMode(true);
       break;
 
-    case 'e':
+    case 'i':
+      fstp.setIgnoredChars(true);
+      fstp.parseICX(optarg);
+      break;
+
+    case 'r':
+      fstp.setRestoreChars(true);
+      fstp.parseRCX(optarg);
+
+    case 'I':
+      fstp.setUseDefaultIgnoredChars(false);
+
+      break;
+
+    case 'e':      
     case 'a':
     case 'b':
     case 'o':
@@ -146,6 +172,7 @@ int main(int argc, char *argv[])
     case 'p':
     case 't':
     case 's':
+    case 'C':
       if(cmd == 0)
       {
 	cmd = c;
@@ -183,21 +210,21 @@ int main(int argc, char *argv[])
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
     input = fopen(argv[optind+1], "rb");
     if(input == NULL || ferror(input))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind+1] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind+1] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
     output= fopen(argv[optind+2], "wb");
     if(output == NULL || ferror(output))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind+2] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind+2] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
@@ -209,14 +236,14 @@ int main(int argc, char *argv[])
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
     input = fopen(argv[optind+1], "rb");
     if(input == NULL || ferror(input))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind+1] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind+1] << "'." << endl << endl;
       exit(EXIT_FAILURE);
     }
 
@@ -228,7 +255,7 @@ int main(int argc, char *argv[])
     FILE *in = fopen(argv[optind], "rb");
     if(in == NULL || ferror(in))
     {
-      cerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
+      wcerr << "Error: Cannot not open file '" << argv[optind] << "'." << endl << endl;
       exit(EXIT_FAILURE);
      }
     fstp.load(in);
@@ -278,6 +305,12 @@ int main(int argc, char *argv[])
         fstp.generation(input, output, gm_tagged_nm);
         break;
 
+      case 'C':
+        fstp.initGeneration();
+        checkValidity(fstp);
+        fstp.generation(input, output, gm_carefulcase);
+        break;
+
       case 'p':
         fstp.initPostgeneration();
         checkValidity(fstp);
@@ -325,7 +358,7 @@ int main(int argc, char *argv[])
   }
   catch (exception& e)
   {
-    cerr << e.what();
+    wcerr << e.what();
     if (fstp.getNullFlush()) {
       fputwc_unlocked(L'\0', output);
     }
